package com.iohao.mmo.equip.listener;

import com.iohao.mmo.equip.entity.EquipGarbage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.mapping.event.AbstractMongoEventListener;
import org.springframework.data.mongodb.core.mapping.event.BeforeDeleteEvent;
import org.springframework.data.mongodb.core.query.BasicQuery;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author 唐斌
 * @date 2023-08-16
 * @description: 监听删除事件，重写删除装备为逻辑删除
 */
@Component
public class EquipEventListener extends AbstractMongoEventListener<Object> {
    private final MongoTemplate mongoTemplate;
    //声明需要格式化的格式(日期加时间)
    DateTimeFormatter dfDateTime = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    public EquipEventListener(MongoTemplate mongoTemplate) {
        this.mongoTemplate = mongoTemplate;
    }

    @Override
    public void onBeforeDelete(BeforeDeleteEvent<Object> event) {
        Set<String> setString = Stream.of("equip", "equipTemplate")
                .collect(Collectors.toUnmodifiableSet());
        if (event.getType() == null || CollectionUtils.isEmpty(event.getDocument())
                || StringUtils.isBlank(event.getCollectionName()) || !setString.contains(event.getCollectionName())) {
            return;
        }
        // 通过事件中返回的`document`对象查询待删除的数据
        List<Object> objects = mongoTemplate.find(new BasicQuery(event.getDocument()), event.getType());
        if (!CollectionUtils.isEmpty(objects)) {
            // 在数据删除前，将所有数据迁移至`garbage`集合中
            String type = event.getCollectionName();
            String nowStr = LocalDateTime.now().format(dfDateTime);
            List<EquipGarbage> equipGarbageList = objects.stream()
                    .map(o -> new EquipGarbage(null, type, o, nowStr))
                    .collect(Collectors.toList());
            mongoTemplate.insertAll(equipGarbageList);
        }
    }
}
